# -*- coding: utf-8 -*-
#            CONFIGURATION MANAGEMENT OF EDF VERSION
# ======================================================================
# COPYRIGHT (C) 1991 - 2002  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
#
#
# ======================================================================
import types,string
import traceback
from copy import copy
from repr import Repr
myrepr = Repr()
myrepr.maxstring = 100
myrepr.maxother = 100

from Noyau.N_utils import repr_float

# Attention : les classes ASSD,.... peuvent etre surcharges
# dans le package Accas. Il faut donc prendre des prcautions si
# on utilise les classes du Noyau pour faire des tests (isxxxx, ...)
# Si on veut crer des objets comme des CO avec les classes du noyau
# ils n'auront pas les conportements des autres packages (pb!!!)
# Il vaut mieux les importer d'Accas mais problme d'import circulaire,
# on ne peut pas les importer au dbut.
# On fait donc un import local quand c'est ncessaire (peut occasionner
# des pbs de prformance).
from Noyau.N_ASSD import ASSD,assd
from Noyau.N_GEOM import GEOM,geom
from Noyau.N_CO import CO
import Accas
# fin attention

from Extensions import parametre
import I_OBJECT
import CONNECTOR

class MCSIMP(I_OBJECT.OBJECT):

  def GetNomConcept(self):
      p=self
      while p.parent :
         try :
            nomconcept=p.get_sdname()
            return nomconcept
         except:
            try :
               nomconcept= p.object.get_sdname()
               return nomconcept
            except :
               pass
         p=p.parent
      return ""

  def GetText(self):
    """
        Retourne le texte  afficher dans l'arbre reprsentant la valeur de l'objet
        point par self
    """

    if self.valeur == None : 
      return None
    elif type(self.valeur) == types.FloatType : 
      # Traitement d'un flottant isol
      # txt = repr_float(self.valeur)
      # Normalement str fait un travail correct
      txt = str(self.valeur)
      clefobj=self.GetNomConcept()
      if self.jdc.appli.dict_reels.has_key(clefobj):
        if self.jdc.appli.dict_reels[clefobj].has_key(self.valeur):
           txt=self.jdc.appli.dict_reels[clefobj][self.valeur]
    elif type(self.valeur) in (types.ListType,types.TupleType) :
      # Traitement des listes
      txt='('
      i=0
      for val in self.valeur:
        if type(val) == types.FloatType : 
           # CCAR : Normalement str fait un travail correct
           #txt=txt + i*',' + repr_float(val)
           clefobj=self.GetNomConcept()
           if self.jdc.appli.dict_reels.has_key(clefobj):
              if self.jdc.appli.dict_reels[clefobj].has_key(val):
                 txt=txt + i*',' +self.jdc.appli.dict_reels[clefobj][val]
              else :
                 txt=txt + i*',' + str(val)
           else :
              txt=txt + i*',' + str(val)
        elif isinstance(val,ASSD): 
           txt = txt + i*',' + val.get_name()
    #PN
    # ajout du elif
        elif type(val) == types.InstanceType and val.__class__.__name__ in  ('PARAMETRE','PARAMETRE_EVAL'):
      	   txt = txt + i*','+ str(val) 
        else: 
           txt = txt + i*','+ myrepr.repr(val)
        i=1
      txt=txt+')'
    elif isinstance(self.valeur,ASSD): 
      # Cas des ASSD
      txt=self.getval()
    elif type(self.valeur) == types.InstanceType and self.valeur.__class__.__name__ in  ('PARAMETRE','PARAMETRE_EVAL'):
      # Cas des PARAMETRES
      txt=str(self.valeur)
    else:
      # Traitement des autres cas
      txt = myrepr.repr(self.valeur)

    # txt peut etre une longue chaine sur plusieurs lignes.
    # Il est possible de tronquer cette chaine au premier \n et 
    # de limiter la longueur de la chaine a 30 caracteres. Cependant
    # ceci provoque une perte d'information pour l'utilisateur
    # Pour le moment on retourne la chaine telle que
    return txt

  def getval(self):
    """ 
       Retourne une chane de caractre reprsentant la valeur de self 
    """
    val=self.valeur
    if type(val) == types.FloatType : 
      clefobj=self.GetNomConcept()
      if self.jdc.appli.dict_reels.has_key(clefobj):
        if self.jdc.appli.dict_reels[clefobj].has_key(val):
           return self.jdc.appli.dict_reels[clefobj][val]
    if type(val) != types.TupleType :
      try:
        return val.get_name()
      except:
        return val
    else :
      s='( '
      for item in val :
        try :
          s=s+item.get_name()+','
        except:
          s=s+`item`+','
      s=s+' )'
      return s

  def wait_co(self):
    """
        Mthode boolenne qui retourne 1 si l'objet attend un objet ASSD 
        qui n'existe pas encore (type CO()), 0 sinon
    """
    for typ in self.definition.type:
      if type(typ) == types.ClassType :
        if issubclass(typ,CO) :
           return 1
    return 0

  def wait_assd(self):
    """ 
        Mthode boolenne qui retourne 1 si le MCS attend un objet de type ASSD 
        ou driv, 0 sinon
    """
    for typ in self.definition.type:
      if type(typ) == types.ClassType :
        if issubclass(typ,ASSD) and not issubclass(typ,GEOM):
          return 1
    return 0

  def wait_assd_or_geom(self):
    """ 
         Retourne 1 si le mot-cl simple attend un objet de type
         assd, ASSD, geom ou GEOM
         Retourne 0 dans le cas contraire
    """
    for typ in self.definition.type:
      if type(typ) == types.ClassType :
        if typ.__name__ in ("GEOM","ASSD","geom","assd") or issubclass(typ,GEOM) :
          return 1
    return 0

  def wait_geom(self):
    """ 
         Retourne 1 si le mot-cl simple attend un objet de type GEOM
         Retourne 0 dans le cas contraire
    """
    for typ in self.definition.type:
      if type(typ) == types.ClassType :
        if issubclass(typ,GEOM) : return 1
    return 0

  def wait_TXM(self):
    """ 
         Retourne 1 si le mot-cl simple attend un objet de type TXM
         Retourne 0 dans le cas contraire
    """
    for typ in self.definition.type:
      if typ == 'TXM' :return 1
    return 0

  def get_liste_valeurs(self):
    """
    """
    if self.valeur == None:
      return []
    elif type(self.valeur) == types.TupleType:
      return list(self.valeur)
    elif type(self.valeur) == types.ListType:
      return self.valeur
    else:
      return [self.valeur]

  def isoblig(self):
    return self.definition.statut=='o'

  def valid_valeur(self,new_valeur):
      """
        Verifie que la valeur passee en argument (new_valeur) est valide
        sans modifier la valeur courante (evite d'utiliser set_valeur et est plus performant)
      """
      old_valeur=self.valeur
      old_val=self.val
      self.valeur = new_valeur
      self.val = new_valeur
      self.state="modified"
      validite=self.isvalid()
      self.valeur = old_valeur
      self.val = old_valeur
      self.state="modified"
      self.isvalid()
      return validite

  def valid_valeur_partielle(self,new_valeur):
      """
        Verifie que la valeur passee en argument (new_valeur) est partiellement valide
        sans modifier la valeur courante (evite d'utiliser set_valeur et est plus performant)
      """
      old_valeur=self.valeur
      old_val=self.val

      self.valeur = new_valeur
      self.val = new_valeur
      self.state="modified"
      validite=0
      if self.isvalid():
         validite=1
      elif self.definition.validators :
         validite=self.definition.validators.valide_liste_partielle(new_valeur)

      if validite==0:
         min,max=self.get_min_max()
         if len(new_valeur) < min :
            validite=1

      self.valeur = old_valeur
      self.val = old_valeur
      self.state="modified"
      self.isvalid()
      return validite

  def set_valeur(self,new_valeur,evaluation='oui'):
        #print "set_valeur",new_valeur
        self.init_modif()
        self.valeur = new_valeur
        self.val = new_valeur
        if self.definition.position == 'global' : 
           self.etape.deep_update_condition_bloc()
        elif self.definition.position == 'global_jdc' :
           self.jdc.deep_update_condition_bloc()
        else:
           self.parent.update_condition_bloc()
        self.fin_modif()
        return 1

  def eval_valeur(self,new_valeur):
    """
        Essaie d'valuer new_valeur comme une SD, une dclaration Python 
        ou un EVAL: Retourne la valeur value (ou None) et le test de russite (1 ou 0)
    """
    #print "eval_valeur",new_valeur
    sd = self.jdc.get_sd_avant_etape(new_valeur,self.etape)
    #sd = self.jdc.get_contexte_avant(self.etape).get(new_valeur,None)
    if sd :
      return sd,1
    else:
      d={}
      # On veut EVAL avec tous ses comportements. On utilise Accas. Perfs ??
      d['EVAL']=Accas.EVAL
      try :
        objet = eval(new_valeur,d)
        return objet,1
      except Exception:
	itparam=self.cherche_item_parametre(new_valeur)
	if itparam:
	     return itparam,1
        if CONTEXT.debug : traceback.print_exc()
        return None,0

  def cherche_item_parametre (self,new_valeur):
        try:
	  nomparam=new_valeur[0:new_valeur.find("[")]
	  indice=new_valeur[new_valeur.find("[")+1:new_valeur.find("]")]
	  for p in self.jdc.params:
	     if p.nom == nomparam :
	        if int(indice) < len(p.get_valeurs()):
		   itparam=parametre.ITEM_PARAMETRE(p,int(indice))
		   return itparam
	  return None
	except:
	  return None

  def update_concept(self,sd):
    if type(self.valeur) in (types.ListType,types.TupleType) :
       if sd in self.valeur:self.fin_modif()
    else:
       if sd == self.valeur:self.fin_modif()

  def delete_concept(self,sd):
    """ 
        Inputs :
           - sd=concept detruit
        Fonction :
        Met a jour la valeur du mot cle simple suite  la disparition 
        du concept sd
    """
    #print "delete_concept",sd
    if type(self.valeur) == types.TupleType :
      if sd in self.valeur:
        self.init_modif()
        self.valeur=list(self.valeur)
        self.valeur.remove(sd)
        self.fin_modif()
    elif type(self.valeur) == types.ListType:
      if sd in self.valeur:
        self.init_modif()
        self.valeur.remove(sd)
        self.fin_modif()
    else:
      if self.valeur == sd:
        self.init_modif()
        self.valeur=None
        self.val=None
        self.fin_modif()

  def replace_concept(self,old_sd,sd):
    """
        Inputs :
           - old_sd=concept remplac
           - sd=nouveau concept
        Fonction :
        Met a jour la valeur du mot cle simple suite au remplacement 
        du concept old_sd
    """
    #print "replace_concept",old_sd,sd
    if type(self.valeur) == types.TupleType :
      if old_sd in self.valeur:
        self.init_modif()
        self.valeur=list(self.valeur)
        i=self.valeur.index(old_sd)
        self.valeur[i]=sd
        self.fin_modif()
    elif type(self.valeur) == types.ListType:
      if old_sd in self.valeur:
        self.init_modif()
        i=self.valeur.index(old_sd)
        self.valeur[i]=sd
        self.fin_modif()
    else:
      if self.valeur == old_sd:
        self.init_modif()
        self.valeur=sd
        self.val=sd
        self.fin_modif()

  def set_valeur_co(self,nom_co):
      """
          Affecte  self l'objet de type CO et de nom nom_co
      """
      #print "set_valeur_co",nom_co
      step=self.etape.parent
      if nom_co == None or nom_co == '':
         new_objet=None
      else:
         # Avant de crer un concept il faut s'assurer du contexte : step 
         # courant
         sd= step.get_sd_autour_etape(nom_co,self.etape,avec='oui')
         if sd:
            # Si un concept du meme nom existe deja dans la porte de l'tape
            # on ne cre pas le concept
            return 0,"un concept de meme nom existe deja"
         # Il n'existe pas de concept de meme nom. On peut donc le crer 
         # Il faut nanmoins que la mthode NommerSdProd de step gre les 
         # contextes en mode editeur
         # Normalement la mthode  du Noyau doit etre surcharge
         # On dclare l'tape du mot cl comme etape courante pour NommerSdprod
         cs= CONTEXT.get_current_step()
         CONTEXT.unset_current_step()
         CONTEXT.set_current_step(step)
         step.set_etape_context(self.etape)
         new_objet = Accas.CO(nom_co)
         CONTEXT.unset_current_step()
         CONTEXT.set_current_step(cs)
      self.init_modif()
      self.valeur = new_objet
      self.val = new_objet
      self.fin_modif()
      step.reset_context()
      # On force l'enregistrement de new_objet en tant que concept produit 
      # de la macro en appelant get_type_produit avec force=1
      self.etape.get_type_produit(force=1)
      #print "set_valeur_co",new_objet
      return 1,"Concept cr"
	
  def verif_existence_sd(self):
     """
        Vrifie que les structures de donnes utilises dans self existent bien dans le contexte
	avant tape, sinon enlve la rfrence  ces concepts
     """
     #print "verif_existence_sd"
     # Attention : possible probleme avec include
     l_sd_avant_etape = self.jdc.get_contexte_avant(self.etape).values()  
     if type(self.valeur) in (types.TupleType,types.ListType) :
       l=[]
       self.init_modif()
       for sd in self.valeur:
         if isinstance(sd,ASSD) :
	    if sd in l_sd_avant_etape :
	       l.append(sd)
	 else:
	    l.append(sd)
       self.valeur=tuple(l)
       self.fin_modif()
     else:
       if isinstance(self.valeur,ASSD) :
	  if self.valeur not in l_sd_avant_etape :
             self.init_modif()
	     self.valeur = None
             self.fin_modif()
 
  def get_min_max(self):
     """
     Retourne les valeurs min et max admissibles pour la valeur de self
     """
     return self.definition.min,self.definition.max


  def get_type(self):
     """
     Retourne le type attendu par le mot-cl simple
     """
     return self.definition.type

#--------------------------------------------------------------------------------
# PN : ajout pour Salome des methodes suivantes (jusqu aux mthodes surchargees)
#--------------------------------------------------------------------------------
  def get_salome_valeurs(self):
       l=[]
       if not hasattr(self,'list_salome_valeurs'):
           self.list_salome_valeurs=[]
       if self.list_salome_valeurs != [] :
           for val in self.list_salome_valeurs:
                l.append(val)
       return l

  def put_salome_valeurs(self,list):
       self.list_salome_valeurs=[]
       for val in list:
           self.list_salome_valeurs.append(val)

  def add_salome_valeurs(self,val):
      if not hasattr(self,'list_salome_valeurs'):
           self.list_salome_valeurs=[]
      try:
           self.list_salome_valeurs.append(val)
      except :
           try:
              for uneval in val :
                  self.list_salome_valeurs.append(uneval)
           except :
              pass

  def has_salome_valeurs(self):
      if not hasattr(self,'list_salome_valeurs'):
           self.list_salome_valeurs=[]
      if self.list_salome_valeurs != []:
           return true
      else:
           return false

#--------------------------------------------------------------------------------
# PN : fin ajout pour Salome 
#--------------------------------------------------------------------------------
 
#ATTENTION SURCHARGE : toutes les methodes ci apres sont des surcharges du Noyau et de Validation
# Elles doivent etre reintegrees des que possible


  def isvalid(self,cr='non'):
      """
         Cette mthode retourne un indicateur de validit de l'objet de type MCSIMP

           - 0 si l'objet est invalide
           - 1 si l'objet est valide

         Le paramtre cr permet de paramtrer le traitement. Si cr == 'oui'
         la mthode construit galement un comte-rendu de validation
         dans self.cr qui doit avoir t cr pralablement.
      """
      if self.state == 'unchanged':
        return self.valid
      else:
        valid = 1
        v=self.valeur
        #  verification presence
        if self.isoblig() and v == None :
          if cr == 'oui' :
            self.cr.fatal(string.join(("Mot-cl : ",self.nom," obligatoire non valoris")))
          valid = 0

        if v is None:
           valid=0
           if cr == 'oui' :
              self.cr.fatal("None n'est pas une valeur autorise")
        else:
           # type,into ...
	   if v.__class__.__name__ in ('PARAMETRE' , 'EVAL', 'ITEM_PARAMETRE','PARAMETRE_EVAL'):
	      verif_type=1
	   else:
	      verif_type=self.verif_type(val=v,cr=None)
	      # cas des tuples avec un ITEM_PARAMETRE
              if verif_type == 0:
                 if type(v) == types.TupleType :
	           new_val=[]
	           for i in v:
	             if i.__class__.__name__  not in ('PARAMETRE','EVAL', 'ITEM_PARAMETRE','PARAMETRE_EVAL'): 
		          new_val.append(i)
		   if new_val != [] :
		     verif_type=self.verif_type(val=new_val,cr=cr)
		   else :
		     # Cas d une liste de paramtre
		     verif_type= 1
		 else:
	             verif_type=self.verif_type(val=v,cr=cr)
           valid = verif_type*self.verif_into(cr=cr)*self.verif_card(cr=cr)
           #
           # On verifie les validateurs s'il y en a et si necessaire (valid == 1)
           #
           if valid and self.definition.validators and not self.definition.validators.verif(self.valeur):
              if cr == 'oui' :
                 self.cr.fatal(string.join(("Mot-cl : ",self.nom,"devrait avoir ",self.definition.validators.info())))
              valid=0
           # fin des validateurs
           #
	# cas d un item Parametre
	if self.valeur.__class__.__name__ == 'ITEM_PARAMETRE':
	   valid=self.valeur.isvalid()
	   if valid == 0:
              if cr == 'oui' :
		 self.cr.fatal(string.join( repr (self.valeur), " a un indice incorrect"))

        self.set_valid(valid)
        return self.valid
